;	======================================
;	        Stroboskop mit Tiny12
;	======================================

;	-------------------------------
;	Port
;	-------------------------------
;	B0: Output activ low Strobo 1
;	B1: Output activ low Strobo 2
;	B2: Output activ low Rundumlicht 1
;	B3: Output activ low Rundumlicht 2
;	B4: Output activ high Warnblinker

;	-------------------------------
;	Timer
;	-------------------------------
;	Timer0: Interrupt every 0,0017 s (Prescaler CK/64)

;	---------------------------------
;	Standard definitions for ATtiny12
;	---------------------------------
	.include "tn12def.inc"

;	-------------------------------
;	Constant definitions
;	-------------------------------
	.equ	cmax	=28		;time loop duration (*0,0478s)
	.equ	c1max	=20		;end value counter 1
	.equ	c2max	=18		;end value counter 2
	.equ	c3max	=12		;end value counter 3
	.equ	c3um	=c3max/4
	.equ	c4max	=11		;end value counter 4
	.equ	c4um	=c4max/4
	.equ	c5max	=10		;end value counter 5
	.equ	port	=portb
	.equ	pin	=pinb
	.equ	ddr	=ddrb
	.equ	out1	=0		;Strobo 1
	.equ	out2	=1		;Strobo 2
	.equ	out3	=2		;Rundum 1
	.equ	out4	=3		;Rundum 2
	.equ	out5	=4		;Blinker

;	-------------------------------
;	Register definitions
;	-------------------------------
	.def	stor	=r0		;storage variable for LPM
	.def	flags	=r1		;flag storage
	.def	temp	=r16		;temporary storage variable 1
	.def	temp2	=r17		;temporary storage variable 2
	.def	c1	=r18		;counter variable 1
	.def	c2	=r19		;counter variable 2
	.def	c3	=r20		;counter variable 3
	.def	c4	=r21		;counter variable 4
	.def	c5	=r22		;counter variable 5
	.def	waico	=r23		;wait counter

;	---------------------------------
;	Program start, Interrupt vectors
;	---------------------------------
	.cseg
	.org	0x0000

reset:	rjmp	initia			;program start
exint0:	reti
pncnge:	reti
t0ovfl:	rjmp	timer0			;time counter
eerdy:	reti
ancomp:	reti

;Initialize I/O and interrupts
initia:	cli				;disable interrupts
	ldi	temp,0x47
	out	osccal,temp		;calibrate oscillator
	ldi	temp,0x1f
	out	port,temp		;set ports
	out	ddr,temp		;set outputs
	ldi	temp,1<<toie0
	out	timsk,temp		;Timer0 interrupt
	ldi	temp,1<<cs01
	out	tccr0,temp		;Timer0 overflow
	sei				;enable interrupts

main:	inc	c1		;Strobo 1
	cpi	c1,4
	breq	c1on
	cpi	c1,c1max
	brlo	c1off
	clr	c1
c1on:	cbi	port,out1
	rjmp	c2a
c1off:	sbi	port,out1

c2a:	inc	c2		;Strobo 2
	cpi	c2,4
	breq	c2on
	cpi	c2,c2max
	brlo	c2off
	clr	c2
c2on:	cbi	port,out2
	rjmp	c3a
c2off:	sbi	port,out2

c3a:	inc	c3		;Rundum 1
	cpi	c3,c3um
	brne	c3b
	sbi	port,out3
c3b:	cpi	c3,c3max
	brlo	c4a
	clr	c3
	cbi	port,out3

c4a:	inc	c4		;Rundum 2
	cpi	c4,c4um
	brne	c4b
	sbi	port,out4
c4b:	cpi	c4,c4max
	brlo	c5a
	clr	c4
	cbi	port,out4

c5a:	inc	c5		;Blinker
	cpi	c5,c5max
	brlo	wait
	clr	c5
	sbis	port,out5
	rjmp	c5on
	cbi	port,out5
	rjmp	wait
c5on:	sbi	port,out5

wait:	clr	waico		;waiting loop
wait2:	cpi	waico,cmax
	brlo	wait2
	rjmp	main

timer0:	in	flags,sreg	;save flags
	inc	waico
	out	sreg,flags	;restore flags
	reti